Package: DEXSeq
Version: 1.58.0
Title: Inference of differential exon usage in RNA-Seq
Authors@R: 
  c(
    person("Simon", "Anders", email = "sanders@fs.tum.de", role = "aut"),
    person("Alejandro", "Reyes", email = "alejandro.reyes.ds@gmail.com", role = c("aut", "ccp"), 
           comment = "Maintainer until 2026."),
    person("Hugo", "Gruson", email = "hugo.gruson@embl.de", role = "cre")
  )
Imports: BiocGenerics, biomaRt, hwriter, methods, stringr, Rsamtools,
        statmod, geneplotter, genefilter
Depends: BiocParallel, Biobase, SummarizedExperiment, IRanges (>=
        2.5.17), GenomicRanges (>= 1.23.7), DESeq2 (>= 1.39.6),
        AnnotationDbi, S4Vectors (>= 0.23.18)
Suggests: GenomeInfoDb, GenomicFeatures, txdbmaker, pasilla (>=
        0.2.22), BiocStyle, knitr, rmarkdown, testthat,
        pasillaBamSubset, GenomicAlignments, roxygen2, glmGamPoi
Description: The package is focused on finding differential exon usage
        using RNA-seq exon counts between samples with different
        experimental designs. It provides functions that allows the
        user to make the necessary statistical tests based on a model
        that uses the negative binomial distribution to estimate the
        variance between biological replicates and generalized linear
        models for testing. The package also provides functions for the
        visualization and exploration of the results.
License: GPL (>= 3)
URL:
biocViews: ImmunoOncology, Sequencing, RNASeq, DifferentialExpression,
        AlternativeSplicing, DifferentialSplicing, GeneExpression,
        Visualization
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Encoding: UTF-8
Config/pak/sysreqs: libbz2-dev libicu-dev liblzma-dev libpng-dev
        libxml2-dev libssl-dev xz-utils zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:35:31 UTC
RemoteUrl: https://github.com/bioc/DEXSeq
RemoteRef: RELEASE_3_23
RemoteSha: 847ca9b8fec10d109175501051311f6cca1453d7
NeedsCompilation: no
Packaged: 2026-04-29 15:03:47 UTC; root
Author: Simon Anders [aut],
  Alejandro Reyes [aut, ccp] (Maintainer until 2026.),
  Hugo Gruson [cre]
Maintainer: Hugo Gruson <hugo.gruson@embl.de>
Built: R 4.6.0; ; 2026-04-29 15:21:54 UTC; unix
