# Generated by roxygen2: do not edit by hand

export(DEXSeq)
export(DEXSeqDataSet)
export(DEXSeqDataSetFromHTSeq)
export(DEXSeqDataSetFromSE)
export(DEXSeqHTML)
export(DEXSeqResults)
export(`exonIDs<-`)
export(`featureIDs<-`)
export(`geneIDs<-`)
export(`groupIDs<-`)
export(estimateExonFoldChanges)
export(exonIDs)
export(featureCounts)
export(featureIDs)
export(geneIDs)
export(groupIDs)
export(perGeneQValue)
export(plotDEXSeq)
export(sampleAnnotation)
export(testForDEU)
exportClasses(DEXSeqDataSet)
exportClasses(DEXSeqResults)
exportMethods(`$<-`)
exportMethods(`[`)
exportMethods(`colData<-`)
exportMethods(counts)
exportMethods(estimateDispersions)
exportMethods(estimateSizeFactors)
exportMethods(findOverlaps)
exportMethods(plotMA)
exportMethods(subsetByOverlaps)
import(Biobase)
import(BiocGenerics)
import(BiocParallel)
import(DESeq2)
import(GenomicRanges)
import(IRanges)
import(Rsamtools)
import(S4Vectors)
import(SummarizedExperiment)
import(biomaRt)
import(genefilter)
import(geneplotter)
import(hwriter)
import(methods)
import(statmod)
import(stringr)
importFrom(AnnotationDbi,select)
importFrom(grDevices,colorRamp)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,palette.colors)
importFrom(grDevices,rgb)
importFrom(grDevices,svg)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,hist)
importFrom(graphics,layout)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(stats,as.formula)
importFrom(stats,coefficients)
importFrom(stats,model.matrix)
importFrom(stats,relevel)
importFrom(stats,runif)
importFrom(stats,terms)
importFrom(stats,weighted.mean)
importFrom(utils,read.delim)
importFrom(utils,read.table)
